include "console.iol"
include "time.iol"
include "runtime.iol"

type ResponseType: void {
	.id: string
	.content?: undefined
	.timeout?: bool
}

type SP_Type: void {
	.text: string
	.timeout?: int
}

interface _thInterface {
  RequestResponse: spellcheck( SP_Type )( string ) throws TimeoutException
  OneWay: _thReq, _thRes( ResponseType )
}

constants {
	DefaultTimeout = 1000
}

execution{ concurrent }

inputPort TimeoutHandlerService {
	Location: "local"
	Interfaces: _thInterface
}

outputPort Self {
	Location: "local"
	Interfaces: _thInterface
}

  init
{
	getLocalLocation@Runtime()( Self.location )
}

outputPort BingSpell {
	RequestResponse: spellcheck
}

embedded {
  Jolie: "BingSpell.ol" in BingSpell
}

cset { id: ResponseType.id }

main
{
	[ spellcheck( x )( y.content ){
		if( !is_defined( x.timeout ) ) {
		  x.timeout = DefaultTimeout
		};
		req.content << x;
		csets.id = req.id = new;
		_thReq@Self( req );
		_thRes( y );
		if( y.timeout ) {
		  throw( TimeoutException )
		}
	}]{ nullProcess }
  [ _thReq( req ) ]{
		first = true;
		{
			{ // regular operation
				spellcheck@BingSpell( req.content.text )( res );
				synchronized( T ){
					if ( first ){
						println@Console( "returning RESULT" )();
						_thRes@Self( { .id = req.id, .content = res } ); 
						first = false
					}
				} 
			}
			|
			{ // timeout
				println@Console( "Timeout: " + req.content.timeout )();
				sleep@Time( req.content.timeout )(); 
				synchronized( T ){
					if ( first ){ 
						println@Console( "returning TIMEOUT" )();
						_thRes@Self( { .id = req.id, .timeout = true } ); 
						first = false
					}
				} 
			}
		}
	}
}
